`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2019/04/22 19:35:26
// Design Name: 
// Module Name: topmodule
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////

module topmodule(
    clk,
    rst,
    vgaRed,
    vgaGreen,
    vgaBlue,
    hsync,
    vsync
);

    input clk; // clock from the crystal
    input rst;
    output reg [3:0] vgaRed, vgaGreen, vgaBlue;
    output hsync, vsync;


    wire clk_25M, clk_22;

    wire clk_1Hz, clk_100Hz;
    wire [1:0] clk_ssd_ctl;

    wire [2:0] random_signal;
    wire [8:0] position;

    wire valid;
    wire [9:0] h_cnt, v_cnt; // 640 x 480
    wire [11:0] pixel_addr;

    wire [11:0] data;
    wire [11:0] pixel_0;
    wire [11:0] pixel_1;
    wire [11:0] pixel_2;
    wire [11:0] pixel_3;
    wire [11:0] pixel_4;
    wire [11:0] pixel_5;
    wire [11:0] pixel_6;
    

    clock_divisor U0(
    .clk1(clk_25M), 
    .clk(clk), 
    .clk22(clk_22)
    );

    clock_generator U1(
    .clk_1Hz(clk_1Hz), //divided clock output
    .clk_100Hz(clk100Hz),
    .clk_ctl(clk_ssd_ctl), //clock for 7-segment
    .clk(clk),    //clock input
    .rst(rst)  //low-active reset
);

    LFSR_random U3(
    .state(random_signal),
    .position(position),
    .rst(rst)
);


    vga_controller U4(
    .pclk(clk_25M),
    .reset(rst),
    .hsync(hsync),
    .vsync(vsync),
    .valid(valid),
    .h_cnt(h_cnt),
    .v_cnt(v_cnt)
    );

    mem_addr_gen U5(
    .clk(clk_1Hz),
    .rst(rst),
    .h_cnt(h_cnt),
    .v_cnt(v_cnt),
    .pixel_addr(pixel_addr),
    .position(position)
);
    blk_mem_gen_0 P0(
	    .clka(clk_25M),
	    .wea(0),
	    .addra(pixel_addr),
	    .dina(data[11:0]),
	    .douta(pixel_0)
		);

	blk_mem_gen_1 P1(
	    .clka(clk_25M),
	    .wea(0),
	    .addra(pixel_addr),
	    .dina(data[11:0]),
	    .douta(pixel_1)
		); 
		
	blk_mem_gen_2 P2(
	    .clka(clk_25M),
	    .wea(0),
	    .addra(pixel_addr),
	    .dina(data[11:0]),
	    .douta(pixel_2)
		); 
		
	blk_mem_gen_3 P3(
	    .clka(clk_25M),
	    .wea(0),
	    .addra(pixel_addr),
	    .dina(data[11:0]),
	    .douta(pixel_3)
		); 
		
	blk_mem_gen_4 P4(
	    .clka(clk_25M),
	    .wea(0),
	    .addra(pixel_addr),
	    .dina(data[11:0]),
	    .douta(pixel_4)
		); 
		
	blk_mem_gen_5 P5(
	    .clka(clk_25M),
	    .wea(0),
	    .addra(pixel_addr),
	    .dina(data[11:0]),
	    .douta(pixel_5)
		);

	blk_mem_gen_6 P6(
	    .clka(clk_25M),
	    .wea(0),
	    .addra(pixel_addr),
	    .dina(data[11:0]),
	    .douta(pixel_6)
		);
    
    always@*
        if (h_cnt < 192 || h_cnt > 447)
            {vgaRed , vgaGreen , vgaBlue} = 12'h0;
		else if (random_signal == 3'd0)
			if ((v_cnt < position || v_cnt > position + 64) && valid)
				{vgaRed , vgaGreen , vgaBlue} = 12'hfff;
			else if ((h_cnt < 288 || h_cnt > 351) && valid)
				{vgaRed , vgaGreen , vgaBlue} = 12'hfff;
			else
				{vgaRed , vgaGreen , vgaBlue} = pixel_0;
		else if(random_signal == 3'd1)
			if((v_cnt < position || v_cnt > position + 64) && valid)
				{vgaRed , vgaGreen , vgaBlue} = 12'hfff;
			else if((h_cnt < 288 || h_cnt > 351) && valid)
				{vgaRed , vgaGreen , vgaBlue} = 12'hfff;
			else
				{vgaRed , vgaGreen , vgaBlue} = pixel_1;
		else if(random_signal == 3'd2)
			if((v_cnt < position || v_cnt > position + 64) && valid)
				{vgaRed , vgaGreen , vgaBlue} = 12'hfff;
			else if((h_cnt < 288 || h_cnt > 351) && valid)
				{vgaRed , vgaGreen , vgaBlue} = 12'hfff;
			else
				{vgaRed , vgaGreen , vgaBlue} = pixel_2;
		else if(random_signal == 3'd3)
			if((v_cnt < position || v_cnt > position + 64) && valid)
				{vgaRed , vgaGreen , vgaBlue} = 12'hfff;
			else if((h_cnt < 288 || h_cnt > 351) && valid)
				{vgaRed , vgaGreen , vgaBlue} = 12'hfff;
			else
				{vgaRed , vgaGreen , vgaBlue} = pixel_3;
		else if(random_signal == 3'd4)
			if((v_cnt < position || v_cnt > position + 64) && valid)
				{vgaRed , vgaGreen , vgaBlue} = 12'hfff;
			else if((h_cnt < 288 || h_cnt > 351) && valid)
				{vgaRed , vgaGreen , vgaBlue} = 12'hfff;
			else
				{vgaRed , vgaGreen , vgaBlue} = pixel_4;
		else if(random_signal == 3'd5)
			if((v_cnt < position || v_cnt > position + 64) && valid)
				{vgaRed , vgaGreen , vgaBlue} = 12'hfff;
			else if((h_cnt < 288 || h_cnt > 351) && valid)
				{vgaRed , vgaGreen , vgaBlue} = 12'hfff;
			else
				{vgaRed , vgaGreen , vgaBlue} = pixel_5;
		else
			if((v_cnt < position || v_cnt > position + 64) && valid)
				{vgaRed , vgaGreen , vgaBlue} = 12'hfff;
			else if((h_cnt < 288 || h_cnt > 351) && valid)
				{vgaRed , vgaGreen , vgaBlue} = 12'hfff;
			else
				{vgaRed , vgaGreen , vgaBlue} = pixel_6;
endmodule



